@@ -928,7 +928,7 @@ bool MainWindow::eventFilter(QObject *watched, QEvent *event)
 		} else if (watched == ui->playlist->viewport()) {
 			destination = ui->playlist;
 			jog_destination = JOG_PLAYLIST;
-			if (destination->columnAt(wheel->x()) != int(PlayList::Column::CAMERA)) {
+			if (destination->columnAt(wheel->position().x()) != int(PlayList::Column::CAMERA)) {
 				last_mousewheel_camera_row = -1;
 			}
 		} else {
@@ -936,8 +936,8 @@ bool MainWindow::eventFilter(QObject *watched, QEvent *event)
 			return false;
 		}
 
-		int column = destination->columnAt(wheel->x());
-		int row = destination->rowAt(wheel->y());
+		int column = destination->columnAt(wheel->position().x());
+		int row = destination->rowAt(wheel->position().y());
 		if (column == -1 || row == -1)
 			return false;
 
