@@ -1,15 +1,18 @@
 #include "alsa_input.h"
 
 #include <alsa/error.h>
+#include <alsa/global.h>
+#include <alsa/pcm.h>
 #include <assert.h>
+#include <chrono>
 #include <errno.h>
+#include <pthread.h>
 #include <stdio.h>
-#include <unistd.h>
+#include <thread>
 #include <cstdint>
 
 #include "alsa_pool.h"
 #include "bmusb/bmusb.h"
-#include "shared/timebase.h"
 
 using namespace std;
 using namespace std::chrono;
