@@ -1,6 +1,8 @@
 #include "alsa_output.h"
 
 #include <alsa/asoundlib.h>
+#include <alsa/error.h>
+#include <alsa/pcm.h>
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
