@@ -14,10 +14,9 @@
 
 #include "audio_mixer.h"
 #include "decibel.h"
-#include "defs.h"
 #include "input_mapping.h"
 #include "resampling_queue.h"
-#include "shared/timebase.h"
+#include "shared/shared_defs.h"
 
 #define NUM_BENCHMARK_CARDS 4
 #define NUM_WARMUP_FRAMES 100
@@ -52,9 +51,9 @@ void reset_lcgrand()
 
 void callback(float level_lufs, float peak_db,
               std::vector<AudioMixer::BusLevel> bus_levels,
-	      float global_level_lufs, float range_low_lufs, float range_high_lufs,
-	      float final_makeup_gain_db,
-	      float correlation)
+              float global_level_lufs, float range_low_lufs, float range_high_lufs,
+              float final_makeup_gain_db,
+              float correlation)
 {
 	// Empty.
 }
