@@ -7,7 +7,7 @@
 #include <atomic>
 #include <chrono>
 #include <condition_variable>
-#include <deque>
+#include <map>
 #include <memory>
 #include <mutex>
 #include <queue>
@@ -16,9 +16,12 @@
 #include <vector>
 
 #include "DeckLinkAPI.h"
+#include "DeckLinkAPIModes.h"
 #include "DeckLinkAPITypes.h"
 #include "LinuxCOM.h"
 
+#include <bmusb/bmusb.h>
+
 #include "shared/context.h"
 #include "print_latency.h"
 #include "quittable_sleeper.h"
