@@ -6,6 +6,9 @@
 #include <map>
 
 #include "bmusb/bmusb.h"
+#include "DeckLinkAPI.h"
+#include "DeckLinkAPITypes.h"
+#include "DeckLinkAPIDiscovery.h"
 
 class IDeckLinkDisplayModeIterator;
 
