@@ -2,7 +2,9 @@
 
 #include <assert.h>
 #include <bmusb/bmusb.h>
+#include <math.h>
 #include <movit/effect_chain.h>
+#include <movit/image_format.h>
 #include <movit/resource_pool.h>
 #include <stdbool.h>
 #include <stdint.h>
@@ -18,14 +20,25 @@
 #include <map>
 #include <mutex>
 #include <utility>
+#include <vector>
+
+#include <QAction>
+#include <QActionGroup>
+#include <QGL>
+#include <QInputDialog>
+#include <QLineEdit>
+#include <QList>
+#include <QObject>
+#include <QVariant>
 
 #include "audio_mixer.h"
 #include "shared/context.h"
 #include "context_menus.h"
-#include "flags.h"
 #include "mainwindow.h"
 #include "mixer.h"
 #include "shared/ref_counted_gl_sync.h"
+#include "shared/shared_defs.h"
+#include "ycbcr_interpretation.h"
 
 class QMouseEvent;
 
