@@ -1,15 +1,32 @@
 #include "image_input.h"
 
+#include <assert.h>
+#include <chrono>
+#include <condition_variable>
+#include <epoxy/egl.h>
 #include <errno.h>
-#include <movit/flat_input.h>
+#include <fcntl.h>
+#include <map>
+#include <memory>
 #include <movit/image_format.h>
 #include <movit/util.h>
+#include <mutex>
+#include <pthread.h>
+#include <stddef.h>
 #include <stdint.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <string>
+#include <sys/stat.h>
+#include <thread>
+#include <time.h>
+#include <utility>
+#include <vector>
 
 extern "C" {
+#include <libavcodec/codec.h>
+#include <libavcodec/codec_par.h>
 #include <libavcodec/avcodec.h>
 #include <libavformat/avformat.h>
 #include <libavutil/avutil.h>
@@ -21,21 +38,11 @@ extern "C" {
 #include <libswscale/swscale.h>
 }
 
-#include <epoxy/egl.h>
-#include <fcntl.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <cstddef>
-#include <functional>
-#include <mutex>
-#include <thread>
-#include <utility>
-#include <vector>
-
+#include "ffmpeg_util.h"
 #include "shared/context.h"
 #include "shared/ffmpeg_raii.h"
-#include "ffmpeg_util.h"
-#include "flags.h"
+#include "shared/ref_counted_texture.h"
+#include "tweaked_inputs.h"
 
 struct SwsContext;
 
@@ -135,8 +142,6 @@ shared_ptr<const ImageInput::Image> ImageInput::load_image_raw(const string &pat
 		fprintf(stderr, "%s: Cannot open decoder\n", pathname.c_str());
 		return nullptr;
 	}
-	unique_ptr<AVCodecContext, decltype(avcodec_close)*> codec_ctx_cleanup(
-		codec_ctx.get(), avcodec_close);
 
 	// Read packets until we have a frame or there are none left.
 	int frame_finished = 0;
