@@ -2,11 +2,9 @@
 #define _IMAGE_INPUT_H 1
 
 #include <epoxy/gl.h>
-#include <movit/flat_input.h>
 #include <stdbool.h>
 #include <time.h>
 #include <condition_variable>
-#include <cstdint>
 #include <map>
 #include <memory>
 #include <mutex>
