@@ -1,12 +1,11 @@
 #include "input_mapping.h"
 
 #include <assert.h>
-#include <fcntl.h>
-#include <google/protobuf/io/zero_copy_stream_impl.h>
-#include <google/protobuf/text_format.h>
-#include <stdio.h>
+#include <map>
 #include <set>
+#include <string>
 #include <utility>
+#include <vector>
 
 #include "audio_mixer.h" 
 #include "state.pb.h"
