@@ -15,13 +15,14 @@
 #include <QTableWidget>
 #include <QVariant>
 #include <functional>
-#include <memory>
 #include <set>
 #include <string>
 #include <utility>
 
 #include "alsa_pool.h"
+#include "audio_mixer.h"
 #include "defs.h"
+#include "input_mapping.h"
 #include "shared/post_to_main_thread.h"
 #include "ui_input_mapping.h"
 
@@ -273,7 +274,7 @@ void InputMappingDialog::remove_clicked()
 void InputMappingDialog::updown_clicked(int direction)
 {
 	assert(ui->table->selectedRanges().size() == 1);
-	const QTableWidgetSelectionRange &range = ui->table->selectedRanges()[0];
+	QTableWidgetSelectionRange range = ui->table->selectedRanges()[0];
 	int a_row = range.bottomRow();
 	int b_row = range.bottomRow() + direction;
 
