@@ -1,22 +1,16 @@
 #include "midi_mapper.h"
 
-#include <alsa/asoundlib.h>
 #include <assert.h>
-#include <errno.h>
-#include <fcntl.h>
 #include <google/protobuf/descriptor.h>
-#include <google/protobuf/io/zero_copy_stream_impl.h>
 #include <google/protobuf/message.h>
-#include <google/protobuf/text_format.h>
-#include <pthread.h>
+#include <map>
+#include <mutex>
 #include <poll.h>
 #include <stdint.h>
 #include <stdio.h>
-#include <sys/eventfd.h>
-#include <unistd.h>
+#include <string>
 #include <algorithm>
 #include <functional>
-#include <thread>
 
 #include "audio_mixer.h"
 #include "nageru_midi_mapping.pb.h"
@@ -55,7 +49,7 @@ void MIDIMapper::set_midi_mapping(const MIDIMappingProto &new_mapping)
 	}
 
 	num_controller_banks = min(max(mapping_proto->num_controller_banks(), 1), 5);
-        current_controller_bank = 0;
+	current_controller_bank = 0;
 
 	receiver->clear_all_highlights();
 	update_highlights();
