@@ -6,21 +6,27 @@
 #include <QComboBox>
 #include <QDialogButtonBox>
 #include <QFileDialog>
+#include <QList>
 #include <QMessageBox>
 #include <QPushButton>
 #include <QSpinBox>
 #include <QStringList>
 #include <QTreeWidget>
+#include <memory>
 #include <stdio.h>
 #include <algorithm>
 #include <cstddef>
 #include <functional>
 #include <limits>
 #include <string>
+#include <utility>
+#include <vector>
 
-#include "shared/controller_spin_box.h"
 #include "midi_mapper.h"
+#include "midi_mapping.pb.h"
 #include "nageru_midi_mapping.pb.h"
+#include "shared/controller_spin_box.h"
+#include "shared/midi_device.h"
 #include "shared/midi_mapper_util.h"
 #include "shared/post_to_main_thread.h"
 #include "ui_midi_mapping.h"
@@ -152,7 +158,7 @@ int get_light_mapping(const MIDIMappingProto &mapping_proto, size_t bus_idx, int
 
 MIDIMappingDialog::MIDIMappingDialog(MIDIMapper *mapper)
 	: ui(new Ui::MIDIMappingDialog),
-          mapper(mapper)
+	  mapper(mapper)
 {
 	ui->setupUi(this);
 
