@@ -14,6 +14,7 @@
 #include <chrono>
 #include <condition_variable>
 #include <cstddef>
+#include <deque>
 #include <functional>
 #include <map>
 #include <memory>
@@ -21,14 +22,14 @@
 #include <queue>
 #include <string>
 #include <thread>
+#include <utility>
 #include <vector>
 
 #include <movit/effect.h>
-#include <movit/image_format.h>
 
 #include "audio_mixer.h"
+#include "card_type.h"
 #include "bmusb/bmusb.h"
-#include "defs.h"
 #include "ffmpeg_capture.h"
 #include "shared/httpd.h"
 #include "input_state.h"
@@ -39,6 +40,7 @@
 #include "shared/ref_counted_gl_sync.h"
 #include "srt_metrics.h"
 #include "theme.h"
+#include "shared/shared_defs.h"
 #include "shared/timebase.h"
 #include "video_encoder.h"
 #include "ycbcr_interpretation.h"
@@ -476,9 +478,6 @@ private:
 
 		std::unique_ptr<PBOFrameAllocator> frame_allocator;
 
-		// Stuff for the OpenGL context (for texture uploading).
-		QSurface *surface = nullptr;
-
 		struct NewFrame {
 			RefCountedFrame frame;
 			int64_t length;  // In TIMEBASE units.
