@@ -1,25 +1,24 @@
 #include "quicksync_encoder.h"
 
+#include <atomic>
+#include <errno.h>
+#include <epoxy/egl.h>
+#include <epoxy/gl.h>
 #include <movit/image_format.h>
 #include <movit/resource_pool.h>  // Must be above the Xlib includes.
 #include <movit/util.h>
 
 #include <EGL/eglplatform.h>
-#include <X11/Xlib.h>
 #include <assert.h>
 #include <epoxy/egl.h>
-#include <fcntl.h>
 #include <glob.h>
 #include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
 #include <va/va.h>
-#include <va/va_drm.h>
 #include <va/va_drmcommon.h>
 #include <va/va_enc_h264.h>
-#include <va/va_x11.h>
 #include <algorithm>
 #include <chrono>
 #include <condition_variable>
@@ -30,31 +29,36 @@
 #include <memory>
 #include <mutex>
 #include <queue>
-#include <stack>
 #include <string>
 #include <thread>
 #include <utility>
+#include <vector>
 
 extern "C" {
 
-#include <libavcodec/avcodec.h>
+#include <drm_fourcc.h>
+#include <libavcodec/packet.h>
+#include <libavformat/avformat.h>
 #include <libavformat/avio.h>
 #include <libavutil/error.h>
-#include <drm_fourcc.h>
 
 }  // namespace
 
 #include "audio_encoder.h"
-#include "shared/context.h"
 #include "defs.h"
-#include "shared/disk_space_estimator.h"
-#include "shared/ffmpeg_raii.h"
 #include "flags.h"
-#include "shared/mux.h"
 #include "print_latency.h"
 #include "quicksync_encoder_impl.h"
 #include "ref_counted_frame.h"
+#include "shared/context.h"
+#include "shared/disk_space_estimator.h"
+#include "shared/ffmpeg_raii.h"
+#include "shared/metrics.h"
+#include "shared/mux.h"
+#include "shared/ref_counted_gl_sync.h"
 #include "shared/timebase.h"
+#include "shared/va_display.h"
+#include "v4l_output.h"
 #include "x264_encoder.h"
 
 using namespace movit;
