@@ -38,14 +38,13 @@
 //  You should have received a copy of the GNU General Public License
 //  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+#include <stddef.h>
 #include <sys/types.h>
 #include <zita-resampler/vresampler.h>
 #include <chrono>
 #include <deque>
-#include <memory>
+#include <string>
 
-#include "defs.h"
-#include "input_mapping.h"
 
 class ResamplingQueue {
 public:
