@@ -3,6 +3,10 @@
 
 // Some tweaked variations of Movit inputs.
 
+#include <string>
+
+#include <epoxy/gl.h>
+#include <movit/image_format.h>
 #include <movit/flat_input.h>
 #include <movit/ycbcr_input.h>
 
