@@ -8,6 +8,7 @@
 #include <string.h>
 
 #include <string>
+#include <x264.h>
 
 using namespace std;
 
@@ -57,7 +58,7 @@ X264Dynamic load_x264_for_bit_depth(unsigned depth)
 			x264_suffix = string(ptr, (m->l_name + strlen(m->l_name)) - ptr);
 			break;
 		}
-        }
+	}
 	dlclose(handle);
 
 	if (x264_dir.empty()) {
