@@ -27,12 +27,6 @@ private:
 		return glFenceSync(condition, flags);
 	}
 
-	static void locked_glDeleteSync(GLsync sync)
-	{
-		std::lock_guard<std::mutex> lock(fence_lock);
-		glDeleteSync(sync);
-	}
-
 	static std::mutex fence_lock;
 };
 
